% anova_ex2 - create the bar chart and do the ANOVA for the data in the
% example data file

% See Figure 6.3b, Figure 6.7b, and Figure 6.8 in HCIERP2e

clear; clc; 
close all; 

path = '.';
fileName = 'anova-ex2';
fileExt = '.txt';

inputDataFile = fullfile(path, [fileName, fileExt]);
M = readmatrix(inputDataFile);

% task completion time results
tMean = mean(M);
tSD = std(M);

barColor = [177, 224, 229] / 255;

f1 = figure;
set(f1, 'color', 'w', 'units', 'normalized', 'position', [.4 .3 .2 .4]);

b = bar(tMean);
b.FaceColor = barColor;
b.EdgeColor = barColor / 4;
b.LineWidth = 1;
ax = gca; 
ax.TickLength = [0 0];
ax.YGrid = 'on';
ax.XTickLabels = {'A', 'B'};
ax.FontSize = 14;

ax.XLabel.String = 'Interaction Method';
ax.XLabel.FontSize = 16;
% ax.XLabel.FontWeight = 'bold';

ax.YLabel.String = 'Task Completion Time (s)';
ax.YLabel.FontSize = 14;
% ax.YLabel.FontWeight = 'bold';
ax.YAxis.TickLabelFormat = '%.1f';
ax.YLim = [0 10];
ax.YTick = 0:10;
ax.YTickLabels = compose('%d', 0:10);

% add data values in bars
s = compose('%.1f', tMean);
text(b.XData, b.YData-.75, s, 'HorizontalAlignment', 'center', ...
    'FontSize', 14);

% add annotation
c1 = [.95 .95 .95]; % background color
c2 = [.7 0 0];   % text color
rectangle('Position',[.03 1.1 2.98 1.4], 'EdgeColor',c2,'LineWidth',2,'FaceColor',c1);
a1 = text(0.1, 1.8, '   Difference is    ','FontSize',16);
a2 = text(1.55, 1.8, '{\itnot significant}   ','FontSize',16,'Color', c2);

exportgraphics(f1, [fileName, 'c.png'], 'Resolution',600);

% save directly in texlive images folder with 2e working filename
% path = 'C:\Users\mack\Desktop\Scott-new\Book2ndEdition\TeXlive_HCI_ERP2\images\';
% file = 'ch6-2ed-03b.png';
% exportgraphics(f1, [path file], 'Resolution',600);

% ====================
% create 2nd bar chart
% ====================

f2 = figure;
set(f2, 'color', 'w', 'units', 'normalized', 'position', [.45 .25 .2 .4]);

b = bar(tMean);
b.FaceColor = barColor;
b.EdgeColor = barColor / 4;
b.LineWidth = 1;
ax = gca; 
ax.TickLength = [0 0];
ax.YGrid = 'on';
ax.XTickLabels = {'A', 'B'};
ax.FontSize = 14;

ax.XLabel.String = 'Interaction Method';
ax.XLabel.FontSize = 16;
% ax.XLabel.FontWeight = 'bold';

ax.YLabel.String = 'Task Completion Time (s)';
ax.YLabel.FontSize = 14;
% ax.YLabel.FontWeight = 'bold';
ax.YAxis.TickLabelFormat = '%.1f';
ax.YLim = [0 10];
ax.YTick = 0:10;
ax.YTickLabels = compose('%d', 0:10);

hold on;

% add error bars
errorbar(b.XData, b.YData, tSD, tSD, 'linestyle', 'none', 'color', 'k', 'linewidth', .8);

% add data values in bars
s = compose('%.1f', tMean);
text(b.XData, [1.5 1.5], s, 'HorizontalAlignment', 'center', ...
    'FontSize', 14);

exportgraphics(f2, [fileName, 'd.png'], 'Resolution',600);

% save directly in texlive images folder with 2e working filename
% path = 'C:\Users\mack\Desktop\Scott-new\Book2ndEdition\TeXlive_HCI_ERP2\images\';
% file = 'ch6-2ed-07b.png';
% exportgraphics(f2, [path file], 'Resolution',600);

% ============
% Do the ANOVA
% ============

T = array2table(M);
T.Properties.VariableNames = {'v1', 'v2'};

withinDesign = table([1 2]', 'VariableNames',{'Interaction_Method'});
withinDesign.Interaction_Method = categorical(withinDesign.Interaction_Method);
rm = fitrm(T, 'v1-v2 ~ 1', 'WithinDesign', withinDesign);

% FYI...
%mTest = mauchly(rm)

AT = ranova(rm, 'WithinModel', 'Interaction_Method');

outFile = strcat(fileName, '_output.txt');
if isfile(outFile)
    delete(outFile);
end
diary(outFile)
disp(anovaTable(AT, 'Task Completion Time (s)', [fileName, fileExt]));
diary off

% -------------------------------------------------------------------------
% Scott's function to create a conventional ANOVA table from the
% overly-complicated and confusing anova table created by the ranova
% function.

function [s] = anovaTable(AT, dvName, dataFile)

c = table2cell(AT);

% remove erroneous entries in F and p columns
for i=1:size(c,1)
    if c{i,4} == 1
        c(i,4) = {''};
    end
    if c{i,5} == .5
        c(i,5) = {''};
    end
end

% use conventional labels in Effect column
effect = AT.Properties.RowNames;
for i=1:length(effect)
    tmp = effect{i};
    tmp = erase(tmp, '(Intercept):');
    tmp = strrep(tmp, 'Error', 'Participant');
    effect(i) = {tmp};
end

% determine the required width of the table
fieldWidth1 = max(cellfun('length', effect)); % width of Effect column
fieldWidth2 = 57; % width for df, SS, MS, F, and p columns

barDouble = repmat('=', 1, fieldWidth1 + fieldWidth2);
barSingle = repmat('-', 1, fieldWidth1 + fieldWidth2);

% re-organize the data
c = c(2:end,[2 1 3 4 5]);
c = [num2cell(repmat(fieldWidth1, size(c,1), 1)), effect(2:end), c]';

% create the ANOVA table
s = sprintf('ANOVA table for %s\n', dvName);
s = [s sprintf('%s\n', barDouble)];
s = [s sprintf('%-*s %4s %11s %14s %9s %9s\n', fieldWidth1, 'Effect', 'df', 'SS', 'MS', 'F', 'p')];
s = [s sprintf('%s\n', barSingle)];
s = [s sprintf('%-*s %4d %14.5f %14.5f %10.3f %10.4f\n', c{:})];
s = [s sprintf('%s\n', barDouble)];
s = [s sprintf('File: %s\n', dataFile)];

end

% *** end ***