function [chi2, df, p, phTable] = chi2test(M, phOption)
%chi2test - perform chi-square test on counts in contingency table
%
%   [chi2, df, p, phTable] = chi2test(M, phOption) returns the chi-square
%   statistic (chi2), the degrees of freedom (df), and the probability (p)
%   for the data in M.  M is a contingency table containing the counts
%   corresponding for the nominal attributes represented in the rows and
%   columns.
%
%   If phOption = 'on', a string (phTable) is returned containing a post
%   hoc pairwise comparisons table for the columns in M.  The comparisons
%   are performed using contrasts, as described by Glass and Hopkins
%   (1984), pp. 391-392.

observed = M;

colTotal = sum(observed,1);
rowTotal = sum(observed,2);
grandTotal = sum(observed, 'all');

nCol = length(colTotal);
nRow = length(rowTotal);

% compute expected counts from observed counts
expected = rowTotal * colTotal ./ grandTotal;

% compute table of chi squares
chisquareTable = (observed-expected).^2 ./ expected;

% compute chi square statistic
chi2 = sum(chisquareTable, 'all');

% compute degrees of freedom
df = (nRow-1) * (nCol-1);

% do chi square test and compute p
p = 1 - chi2cdf(chi2, df);

% we're done, except for the possibility of pairwise comparisons
phTable = [];
if strcmp(phOption,'on')

    % -----------------------------------------------------------------
    % do post hoc pairwise comparisons (as per ChiSquareGUI in GoStats)
    % -----------------------------------------------------------------
    % source: Glass and Hopkins (1984), pp. 391-392

    sigma = (observed(1,:) ./ colTotal .* (1 - observed(1,:) ./ colTotal)) ./ colTotal;

    pair = nchoosek(1:nCol,2);
    nPair = size(pair,1);
    s = newline;
    for k=1:nPair
        i = pair(k,1);
        j = pair(k,2);
        contrast = observed(1,i) / colTotal(i) - observed(1,j) / colTotal(j);
        chi22 = contrast * contrast / (sigma(i) + sigma(j));
        p = 1 - chi2cdf(chi22, df);
        if p < 0.05
            pString = '*';
        else
            pString = '';
        end
        s = [s, sprintf('Pair %d:%d  --->  Chi-square(%d) = %6.3f, p = %5.4f%s\n', ...
            i, j, df, chi2, p,pString)];
    end

    % print results summary (like in GoStats)

    phTable = [phTable sprintf('---------------------------------------------------\n')];
    phTable = [phTable sprintf('------ Pairwise Comparisons (using contrasts) -----\n')];
    phTable = [phTable sprintf('---------------------------------------------------\n')];
    phTable = [phTable sprintf('%s', s)];
    phTable = [phTable sprintf('---------------------------------------------------\n')];
    phTable = [phTable sprintf('* = significant (alpha = .05)\n')];

end

end