% dix_example_10x2 - create the bar chart and do the ANOVA for the data in
% the example data file

% See Figure 6.18 in HCIERP2e

clear; clc; 
close all; 

path = '.'; % use this path after extracting from ZIP file

fileName = 'dix-example-10x2';
fileExt = '.txt';

inputDataFile = fullfile(path, [fileName, fileExt]);
M = readmatrix(inputDataFile);

% task completion time results
tMean = mean(M);
tSD = std(M);

barColor = [177, 224, 229] / 255;

% =================================
% create bar chart (not in HCI:ERP2)
% ==================================

f = figure;
set(f, 'color', 'w', 'units', 'normalized', 'position', [.4 .3 .2 .4]);

b = bar(tMean);
b.FaceColor = barColor;
b.EdgeColor = barColor / 4;
b.LineWidth = 1;
ax = gca; 
ax.TickLength = [0 0];
ax.YGrid = 'on';
ax.XTickLabels = {'Natural', 'Abstract'};
ax.FontSize = 14;

ax.XLabel.String = 'Icon Type';
ax.XLabel.FontSize = 16;
% ax.XLabel.FontWeight = 'bold';

ax.YLabel.String = 'Task Completion Time (s)';
ax.YLabel.FontSize = 14;
% ax.YLabel.FontWeight = 'bold';
ax.YAxis.TickLabelFormat = '%.1f';
ax.YLim = [0 1200];
ax.YTick = 0:200:1200;
ax.YTickLabels = compose('%d', 0:200:1200);

hold on;

% add error bars
errorbar(b.XData, b.YData, tSD, tSD, 'linestyle', 'none', 'color', 'k', 'linewidth', .8);

% add data values in bars
s = compose('%.0f', tMean);
text(b.XData, b.YData/2, s, 'HorizontalAlignment', 'center', 'FontSize', 14);

exportgraphics(f, [fileName, '.png']);

% ============
% Do the ANOVA
% ============

T = array2table(M);
T.Properties.VariableNames = {'v1', 'v2'};

withinDesign = table([1 2]', 'VariableNames',{'Icon_Type'});
withinDesign.Icon_Type = categorical(withinDesign.Icon_Type);
rm = fitrm(T, 'v1-v2 ~ 1', 'WithinDesign', withinDesign);

% FYI...
%mTest = mauchly(rm)

AT = ranova(rm, 'WithinModel', 'Icon_Type');

outFile = strcat(fileName, '_output.txt');
if isfile(outFile)
    delete(outFile);
end
diary(outFile)
disp(anovaTable(AT, 'Task Completion Time (s)', [fileName, fileExt]));
diary off

% -------------------------------------------------------------------------
% Scott's function to create a conventional ANOVA table from the
% overly-complicated and confusing anova table created by the ranova
% function.

function [s] = anovaTable(AT, dvName, dataFile)

c = table2cell(AT);

% remove erroneous entries in F and p columns
for i=1:size(c,1)
    if c{i,4} == 1
        c(i,4) = {''};
    end
    if c{i,5} == .5
        c(i,5) = {''};
    end
end

% use conventional labels in Effect column
effect = AT.Properties.RowNames;
for i=1:length(effect)
    tmp = effect{i};
    tmp = erase(tmp, '(Intercept):');
    tmp = strrep(tmp, 'Error', 'Participant');
    effect(i) = {tmp};
end

% determine the required width of the table
fieldWidth1 = max(cellfun('length', effect)); % width of Effect column
fieldWidth2 = 57; % width for df, SS, MS, F, and p columns

barDouble = repmat('=', 1, fieldWidth1 + fieldWidth2);
barSingle = repmat('-', 1, fieldWidth1 + fieldWidth2);

% re-organize the data
c = c(2:end,[2 1 3 4 5]);
c = [num2cell(repmat(fieldWidth1, size(c,1), 1)), effect(2:end), c]';

% create the ANOVA table
s = sprintf('ANOVA table for %s\n', dvName);
s = [s sprintf('%s\n', barDouble)];
s = [s sprintf('%-*s %4s %11s %14s %9s %9s\n', fieldWidth1, 'Effect', 'df', 'SS', 'MS', 'F', 'p')];
s = [s sprintf('%s\n', barSingle)];
s = [s sprintf('%-*s %4d %14.5f %14.5f %10.3f %10.4f\n', c{:})];
s = [s sprintf('%s\n', barDouble)];
s = [s sprintf('File: %s\n', dataFile)];

end

% *** end ***