function [h, p, phTable] = friedman2(M, phOption)
%friedman2 - perform the Friedman test on a matrix of data
%
%   [h, p, phTable] = friedman2(M, phOption) returns the test statistic (h)
%   and the probability (p) for a Friedman test performed on the data in
%   the matrix M. The values for h and p are corrected for ties.
%
%   If phOption = 'on', a string (phTable) is returned containing a post
%   hoc pairwise comparisons table for the columns in M.  The comparisons
%   are performed using the Bonferroni-Dunn method, as described by Sheskin
%   (2011), p. 1100.

% The Friedman test is a non-parametric test comparing three or more
% sets of ordinal (i.e., ranked) data.  The sets are 'dependent', meaning
% they are from a group of participants with each participant tested on all
% conditions.  This is also known as a within-subjects design.
%
% The computations below are from Test 25 (pp. 1095-1113) in Sheskin's
% "Handbook of Parametric and Non-parametric Statistical Procedures", 5th
% ed., CRC Press, 2011.
%
% We're using alpha = 0.05.  This includes the familywise alpha for the
% post hoc pairwise comparisons test.

n = size(M,1); % number of rows (participants)
k = size(M,2); % number of columns (conditions)

[R, ~] = tiedrank(M'); % across rows
R = R';

ss = sum(sum(R).^2); % using ranks, across rows

% uncorrected for ties
h1 = (12 * ss) / (n * k * (k + 1)) - 3 * n * (k + 1);
% p1 = chi2cdf(h1, k-1, 'upper');

% see Sheskin, p. 1099
factor = 0;
for i=1:n
    [~, tieadj] = tiedrank(M(i,:));
    factor = factor + 2 * tieadj;
end

cft = 1 - factor / (n * (k^3 - k)); % correction for ties

h = h1 / cft;
p = chi2cdf(h, k-1, 'upper');

% we're done, except for the possibility of pairwise comparisons
phTable = [];
if strcmp(phOption,'on')

    aFW = 0.05;        % familywise alpha
    c = nchoosek(k,2); % number of comparisons
    aPC = aFW / (1*c); % per-comparison alpha
    z = norminv(aPC);

    % critical difference
    cd = abs(z * sqrt((n * k * (k+1))/6));

    pairs = nchoosek(1:k,2);
    nPairs = size(pairs,1);
    groupMeans = sum(R);

    for i=1:nPairs
        pairDifference(i,1) = abs(diff(groupMeans(pairs(i,:))))';
    end

    sigDiff = repelem({'-'}, size(pairs,1), 1);
    sigDiff(pairDifference > cd) = {'* (significant)'};

    % organize data in cell array to simplify printing
    C = [num2cell([pairs, pairDifference, repelem(cd,nPairs,1)]), sigDiff]';

    ruler = '------------------------------------------------------------';
    ruler2 = ruler;
    phTable = sprintf(' Pairwise Comparisons (%s) ', 'Bonferroni-Dunn');
    n = floor(length(ruler)/2 - length(phTable)/2);
    ruler2(n:(n+length(phTable)-1)) = phTable;

    phTable = sprintf('%s\n', ruler);
    phTable = [phTable sprintf('%s\n', ruler2)];
    phTable = [phTable sprintf('%s\n', ruler)];
    phTable = [phTable sprintf('Pair  %d:%d  -->  %8.3f    > %8.3f   ?   %s\n', C{:})];
    phTable = [phTable sprintf('%s\n', ruler)];

end

end

% *** end ***