function [h, p, phTable] = kruskalwallis2(M, phOption)
%kruskalwallis2 - perform the Kruskal-Wallis test on a matrix of data
%
%   [h, p, pwTable] = kruskalwallis2(M, phOption) returns the test
%   statistic (h) and the probability (p) for a Kruskal-Wallis test
%   performed on the data in the matrix M. The values for h and p are
%   corrected for ties.
%
%   If phOption = 'on', a string (phTable) is returned containing a
%   pairwise comparisons table for the columns in M.  The comparisons are
%   performed using the Bonferroni-Dunn method, as described by Sheskin
%   (2011), pp. 1006-1007.

% The Kruskal-Wallis test is a non-parametric test comparing three or more
% sets of ordinal (i.e., ranked) data.  The sets are 'independent', meaning
% they are from different groups of participants.  This is also known as a
% between-subjects design.  The groups are ideally the same size but the
% test is also valid if the sets differ in size.

k = size(M,2); % number of conditions

[R, tieadj] = tiedrank(M(:));
R2 = reshape(R, [], k); % matrix of ranks (perhaps including nans)
R(isnan(R)) = [];       % vector of ranks (no nans)
N = length(R);          % total number of observations

% can this be simplified?
for i=1:k
    idx2 = ~isnan(R2(:,i));
    total = sum(R2(idx2,i));
    t2count(i) = total * total / sum(idx2);
end
a = 12 / (N * (N+1));
b = sum(t2count);
c = 3 * (N + 1);

h1 = a * b - c;
p1 = 1 - chi2cdf(h1,k-1);

cft = 1 - 2*tieadj / (N^3 - N); % correction for ties

h = h1 / cft;
p = 1 - chi2cdf(h,k-1);

% we're done, except for the possibility of pairwise comparisons
phTable = [];
if strcmp(phOption,'on')

    % This is the Bonnferroni-Dunn test, but it is a bit more complicated than
    % usual because the group sizes might differ.  This means the critical
    % difference must be calculated separately for each test.

    aFW = 0.05;              % familywise alpha
    pairs = nchoosek(1:k,2); % indices for the pairwise tests
    c = size(pairs,1);       % number of comparisons
    aPC = aFW / (2*c);       % per-comparison alpha
    z = norminv(aPC);

    % critical difference (compute separately for each comparison)
    for i=1:c
        idx1 = pairs(i,1);
        idx2 = pairs(i,2);
        na = length(R2(~isnan(R2(:,idx1)),idx1));
        nb = length(R2(~isnan(R2(:,idx2)),idx2));
        cd(i) = abs(z * sqrt((N*(N+1)/12) * (1/na + 1/nb)));
    end

    groupMeans = mean(R2, 'omitnan');

    k = length(groupMeans);
    pairs = nchoosek(1:k,2);
    nPairs = size(pairs,1);

    sigDiff = repelem({'-'}, size(pairs,1), 1);
    for i=1:nPairs
        pairDifference(i) = abs(diff(groupMeans(pairs(i,:))))';
        if pairDifference(i) > cd(i)
            sigDiff(i) = {'* (significant)'};
        end
    end

    % organize data in cell array to simplify printing
    C = [num2cell([pairs, pairDifference', cd']), sigDiff]';

    ruler = '------------------------------------------------------------';
    ruler2 = ruler;
    s = sprintf(' Pairwise Comparisons (%s) ', 'Bonferroni-Dunn');
    n = floor(length(ruler)/2 - length(s)/2);
    ruler2(n:(n+length(s)-1)) = s;

    phTable = newline;
    phTable = [phTable sprintf('%s\n', ruler)];
    phTable = [phTable sprintf('%s\n', ruler2)];
    phTable = [phTable sprintf('%s\n', ruler)];
    phTable = [phTable sprintf('Pair  %d:%d  -->  %8.3f    > %8.3f   ?   %s\n', C{:})];
    phTable = [phTable sprintf('%s\n', ruler)];
    % pwTable = [pwTable sprintf('Data file: %s\n', file)];

end

end

% *** end ***