function [M, cv, f] = lilliefors(v, displayopt)
%lilliefors perform normality test on vector of data
%
%   Input arguments:
%     v = vector of input data
%     displayopt = 'off' (default) or 'on' (to generate and save chart)
%
%   Return values:
%     M - the test statistic
%     cv - the criticial value
%     f - handle to figure (if displayopt = 'on')

% Below are the critical values for the Lilliefors test for normality. The
% values are for a non-directional (i.e., two-tailed) test at p < .05. The
% data were transcribed from the following source:
%
% Abdi, H. and Molin, P., Lilliefors test for normality, in Encyclopedia of
% Measurement and Statistics, (N. J. Salkind, Ed.). Thousand Oaks, CA:
% Sage, 2007, 540-544.  Available online at...
%
% https://www.real-statistics.com/statistics-tables/lilliefors-test-table/
%
% The lookup table includes critical values for n = 4 to n = 50. For
% samples with n > 50, we use 0.895 / fN, where fN = ((0.83 + n) / SQRT(n))
% - 0.01.
cvLookup = [0.3754 0.3427 0.3245 0.3041 0.2875 0.2744 0.2616 0.2506 0.2426 ...
    0.2337 0.2257 0.2196 0.2128 0.2071 0.2018 0.1965 0.1920 0.1881 ...
    0.1840 0.1798 0.1766 0.1726 0.1699 0.1665 0.1641 0.1614 0.1590 ...
    0.1559 0.1542 0.1518 0.1497 0.1478 0.1454 0.1436 0.1421 0.1402 ...
    0.1386 0.1373 0.1353 0.1339 0.1322 0.1309 0.1293 0.1282 0.1269 ...
    0.1256 0.1246];

y = reshape(v, 1, []); % ensure data are row vector
y = sort(y);
n = length(y); % sample size (original data)

% only need unique values (however, we still need n for all samples)
[y2, ia, ~] = unique(y, 'last');

% Note: variable labels are from Sheskin reference
A = sort(y2);

% z score
B = (A - mean(y)) / std(y); % Lilliefors test uses sample mean and sd

% proportion of cases in normal distribution that fall between mean and z
C = abs(0.5 - normcdf(B));

% cumulative proportion for a given X score in hypothesised distribution
D = 0.5 + C .* sign(B); % ECF (expected cumulative frequency)

E = (ia / n)'; % OCF (observed cumulative frequency)

F = abs(E - D);

G = abs([0 E(1:end-1)] - D);

M = [A' B' C' D' E' F' G']; % just FYI

% compute test statistic
%
% This is complicated because we might also create a plot to visualize the
% result. For the plot, an arrow will be added to show the test statistic.
% We need idx and adjust for that.
[testStatistic1, idx1] = max(F);
[testStatistic2, idx2] = max(G);
if testStatistic1 > testStatistic2
    testStatistic = testStatistic1;
    idx = idx1;
    adjust = 0;
else
    testStatistic = testStatistic2;
    idx = idx2;
    adjust = -1;
end

% compute critical value
if n < 4
    beep;
    fprintf('Test not valid (n=%d)\n', n);
    return;
elseif n <= 50
    criticalValue = cvLookup(n-4+1);
else % n > 50
    fN = ((0.83 + n) / sqrt(n)) - 0.01;
    criticalValue = 0.895 / fN;
end

M = testStatistic;
cv = criticalValue;

% we're done (unless displayopt = 'on')

f = [];
if strcmp(displayopt, 'on')
    f = figure;

    % plot the observed cumulative frequency (stairstep appearance)
    xOCF = repelem(B,2);
    yOCF = [0 repelem(E(1:end-1),2) E(end)];
    plot(xOCF, yOCF, '-b', 'linewidth', 1.5);
    title('Lilliefors Test For Normality', 'fontsize', 12,'FontWeight','normal');
    xlabel('Z');
    ylabel('Cumulative Frequency');
    set(gcf,'color', 'w', 'units', 'normalized', 'position', [.35 .2 .25 .4]);
    set(gca, 'ygrid', 'on', 'xgrid', 'on');

    % add plot for expected cumulative frequency (continuous curve)
    hold on;
    plot(B, D, 'linewidth', 1.5);

    % add an arrow showing the test statistic (this is a neat feature)
    p1 = [xOCF(2*idx+adjust), yOCF(2*idx+adjust)];
    p2 = [B(idx) D(idx)];
    dp = p2 - p1;
    quiver(p1(1), p1(2), dp(1), dp(2), 0, '-k', 'linewidth', 2, 'MaxHeadSize',14);

    % add legend
    labels = {'Observed cumulative frequency (OCF)', ...
        'Expected cumulative frequency (ECF)', 'Test statistic'};
    legend(labels, 'location', 'northwest');

    if testStatistic < criticalValue
        decision = 'Tenable';
    else
        decision = 'Rejected';
    end

    % add summary text
    s = sprintf('Test statistic: %.5f\n', testStatistic);
    s = [s sprintf('Critical value: %.5f\n', criticalValue)];
    s = [s sprintf('Normality hypothesis: %s', decision)];
    ax = gca;
    xPadding = diff(ax.XLim) * 0.04;
    yPadding = diff(ax.YLim) * 0.04;
    x = ax.XLim(2) - xPadding;
    y = ax.YLim(1) + yPadding;
    text(x, y, s, 'backgroundcolor', 'w', ...
        'horizontalalignment', 'right', 'verticalalignment', 'bottom');

    shg;
end

end