function [U, z, p] = mannwhitneyu(M)
%mannwhitneyu - perform the Mann-Whitney U test on two columns of data
%
%   [U, z, p] = mannwhitneyu(M) returns the test statistic (U), the
%   corresponding z-score (z), and the probability (p) for the Mann Whitney
%   U test performed on the data in the first two columns of matrix M.  The
%   values for z and p are corrected for ties.
%
%   The Mann Whitney U test is a non-parametric test comparing two sets of
%   ordinal (i.e., ranked) data.  The sets are 'independent', meaning they
%   are from different groups of participants.  This is also known as a
%   between-subjects design.  The two groups are ideally the same size but
%   the test is also valid if the two sets differ in size.

% The computations below are from pp. 533-539 in Sheskin's "Handbook of
% Parametric and Non-parametric Statistical Procedures", 5th ed., CRC
% Press, 2011.

% MATLAB's ranksum function purports to be the same as the Mann-Whitney U
% test, but gives a different result.  See last line of code in this
% script.  I posted a MATLAB question about this, but did not receive an
% answer.

% extract first two columns from M
x1 = M(:,1);
x2 = M(:,2);

% ensure x1 and x2 are column vectors and without nans (lengths may differ)
x1 = x1(~isnan(x1));
x2 = x2(~isnan(x2));
x1 = reshape(x1,[],1);
x2 = reshape(x2,[],1);

n1 = length(x1);
n2 = length(x2);

[R, tieadj] = tiedrank([x1; x2]);
R2 = reshape(R, [], 2);

U1 = n1 * n2 + (n1 * (n1+1))/2 - sum(R2(:,1));
U2 = n1 * n2 + (n2 * (n2+1))/2 - sum(R2(:,2));
U = min([U1 U2]);

% z1 = (U - n1*n2/2) / sqrt(n1*n2*(n1+n2+1)/12);
% p1 = 2*normcdf(z1);

% compute 'factor' for adjusting for ties (Sheskin, p. 539)
% x = [x1; x2];
% y = countcats(categorical(x));
% factor = sum(y.^3 - y)

% tieadj from the tiedrank function also gives this factor, but /2
factor = tieadj * 2;

z = (U - n1*n2/2) / sqrt((n1*n2*(n1+n2+1)/12) - n1*n2*abs(factor)/(12*(n1+n2)*(n1+n1-1)));
p = 2*normcdf(z);

end