function [mc, phTable] = posthoc(M, type)
%posthoc - perform post hoc pairwise comparisons on data in M
%
%   [mc, phTable] = posthoc(M, type) returns a double matrix (mc) and a
%   string table (phTable) containing the post hoc pairwise comparisons on
%   the columns of data in a matrix (M).  The comparisons 'type' is one of
%   'scheffe', 'bonferroni', 'fisher', or 'tukey'. The comparisons are
%   performed between the columns of data, two columns per comparison.
%   Alpha is fixed at .05.
%
%   The results data matrix contains one row per comparison.  Each row
%   contains five elements.  The first two are the column numbers for the
%   data being compared.  Next is the difference in the means for the
%   columns.  This is followed by the critical difference.  The last entry
%   is the result (0 = not significant; 1 = significant).
%
%   The comparison table is a print-ready version of the data matrix.  Each
%   row contains, in order, the numbers for the columns being compared, the
%   difference in the means for the data in those columns, the critical
%   difference computed according to the selected comparisions type, and
%   the result.  The result is either '-' for not significant or '*' for
%   significant.

alpha = .05;            % default alpha (probability of Type-I error)
k = size(M,2);          % number of columns (conditions)
n = size(M,1);          % number of rows (size of each group)
N = numel(M);           % number of scores (total)
C = nchoosek(1:k,2); % number of pairs
nPairs = size(C,1);

df1 = k-1; % aka 'between-groups df'
df2 = N-k; % aka 'within-groups df'

% sum of squares (total, between-groups, within-groups)
SST = sum(M.^2, 'all') - sum(M, 'all')^2 / N;
SSBG = sum(sum(M).^2)/n - sum(M, 'all')^2 / N;
SSWG = SST - SSBG;

% mean squares
% MSBG = SSBG / df1;
MSWG = SSWG / df2;

switch (type)
    
    % -------------------------------------------------------------------------
    % Scheffe test (Sheskin, p. 913)
    case 'scheffe'  
        testName = ['Scheff' char(233)];
        
        % get critical value for F-statistic
        % F = getF(df1, df2); % Using Scott's function
        % F = finv(1-alpha, df1, df2);
        F = icdf('F', 1-alpha, df1, df2);
        
        % Scheffe critical difference
        criticalDifference = sqrt((k-1)*F) * sqrt(2*MSWG/n);        
        
    % -------------------------------------------------------------------------
    % Fisher's LSD test (Sheskin, p. 903)
    case 'fisher'   
        testName = 'Fisher LSD';
        
        % get critical value for F-statistic
        % F = getF(df1, df2); % using Scott's function
        % F = finv(1-alpha, df1, df2);
        F = icdf('F', 1-alpha, df1, df2);
        
        % Fisher LSD critical difference
        criticalDifference = sqrt(F) * sqrt(2*MSWG/n);        
        
    % -------------------------------------------------------------------------
    % Bonferroni-Dunn (Sheskin, p. 907)
    case 'bonferroni'
        testName = 'Bonferroni-Dunn';
        
        z = abs(norminv(alpha / nPairs / 2));
        t = z + (z^3 + z) / (4 * (df2 - 2));
        
        % Bonferroni-Dunn critical difference
        criticalDifference = t * sqrt(2 * MSWG / n);        
        
    %--------------------------------------------------------------------------
    % Tukey's HSD test (Sheskin, p. 909)
    case 'tukey'
        testName = 'Tukey HSD';
        
        % NOTE: HSD = "honestly significant difference"
        
        % get critical value for Studentized Range statistic
        q = getQ(df1, df2); % custom function written by Scott (alpha = .05)
        
        % Tukey's HSD critical difference
        criticalDifference = q * sqrt(MSWG / n);
        
end

% ------------------------------------------
% make output double matrix and string table
% ------------------------------------------

groupMeans = mean(M);
pairs = nchoosek(1:k,2);
mc = [];
for i=1:nPairs
    pairDifference(i,1) = abs(diff(groupMeans(pairs(i,:))))';
    mc(i,:) = [pairs(i,1), pairs(i,2), pairDifference(i,1), criticalDifference];
end

sigDiff = repelem({'-'}, size(pairs,1), 1);
sigLogical = pairDifference > criticalDifference;
sigDiff(sigLogical) = {'* (significant)'};
mc = [mc, sigLogical];

% organize data in cell array to simplify printing
C = [num2cell([pairs, pairDifference, repelem(criticalDifference,nPairs,1)]), sigDiff]';

ruler = '------------------------------------------------------------';
ruler2 = ruler;
tmp = sprintf(' Pairwise Comparisons (%s) ', testName);
n = floor(length(ruler)/2 - length(tmp)/2);
ruler2(n:(n+length(tmp)-1)) = tmp;

phTable = sprintf('%s\n', ruler);
phTable = [phTable sprintf('%s\n', ruler2)];
phTable = [phTable sprintf('%s\n', ruler)];
phTable = [phTable sprintf('Pair  %d:%d  -->  %8.3f    > %8.3f   ?   %s\n', C{:})];
phTable = [phTable sprintf('%s\n', ruler)];

end

