% posthoc_ex1 - create the bar chart and do the ANOVA for the data in the
% example data file

% See Figure 6.10b, Figure 6.11, and Figure 6.12 in HCIERP2e

clear; clc; 
close all; 

path = '.';
fileName = 'posthoc-ex1';
fileExt = '.txt';

inputDataFile = fullfile(path, [fileName, fileExt]);
M = readmatrix(inputDataFile);

% task completion time results
tMean = mean(M);
tSD = std(M);

barColor = [229, 224, 177] / 255;

f = figure;
set(f, 'color', 'w', 'units', 'normalized', 'position', [.4 .3 .35 .4]);

b = bar(tMean);
b.FaceColor = barColor;
b.EdgeColor = barColor / 4;
b.LineWidth = 1;
b.BarWidth = 0.6;
ax = gca; 
ax.TickLength = [0 0];
ax.YGrid = 'on';
ax.XTickLabels = {'A', 'B', 'C', 'D'};
ax.FontSize = 14;

ax.XLabel.String = 'Interaction Method';
ax.XLabel.FontSize = 16;
% ax.XLabel.FontWeight = 'bold';

ax.YLabel.String = 'Dependent Variable (units)';
ax.YLabel.FontSize = 14;
% ax.YLabel.FontWeight = 'bold';
ax.YAxis.TickLabelFormat = '%.1f';
ax.YLim = [0 25];
ax.YTick = 0:5:25;
ax.YTickLabels = compose('%d', 0:5:25);

hold on;

% add error bars
errorbar(b.XData, b.YData, tSD, tSD, 'linestyle', 'none', 'color', 'k', 'linewidth', .8);

% add data values in bars
s = compose('%.2f', tMean);
text(b.XData, b.YData/2, s, 'HorizontalAlignment', 'center', 'FontSize', 14);

exportgraphics(f, [fileName, '.png'], 'resolution', 600);

% save directly in texlive images folder with 2e working filename
% path = 'C:\Users\mack\Desktop\Scott-new\Book2ndEdition\TeXlive_HCI_ERP2\images\';
% file = 'ch6-2ed-10b.png';
% exportgraphics(f, [path file], 'Resolution',600);

% ============
% Do the ANOVA
% ============

T = array2table(M);
T.Properties.VariableNames = {'v1', 'v2', 'v3', 'v4'};

withinDesign = table([1 2 3 4]', 'VariableNames',{'Interaction_Method'});
withinDesign.Interaction_Method = categorical(withinDesign.Interaction_Method);
rm = fitrm(T, 'v1-v4 ~ 1', 'WithinDesign', withinDesign);

% FYI...
%mTest = mauchly(rm)

AT = ranova(rm, 'WithinModel', 'Interaction_Method');

outFile = strcat(fileName, '_output.txt');
if isfile(outFile)
    delete(outFile);
end
diary(outFile)

disp(anovaTable(AT, 'Dependent Variable (units)', [fileName, fileExt]));

% do the post hoc test using Scott's custom MATLAB function
[~, s] = posthoc(M, 'scheffe');
fprintf('%s\n', s);
diary off

% -------------------------------------------------------------------------
% Scott's function to create a conventional ANOVA table from the
% overly-complicated and confusing anova table created by the ranova
% function.

function [s] = anovaTable(AT, dvName, dataFile)

c = table2cell(AT);

% remove erroneous entries in F and p columns
for i=1:size(c,1)
    if c{i,4} == 1
        c(i,4) = {''};
    end
    if c{i,5} == .5
        c(i,5) = {''};
    end
end

% use conventional labels in Effect column
effect = AT.Properties.RowNames;
for i=1:length(effect)
    tmp = effect{i};
    tmp = erase(tmp, '(Intercept):');
    tmp = strrep(tmp, 'Error', 'Participant');
    effect(i) = {tmp};
end

% determine the required width of the table
fieldWidth1 = max(cellfun('length', effect)); % width of Effect column
fieldWidth2 = 57; % width for df, SS, MS, F, and p columns

barDouble = repmat('=', 1, fieldWidth1 + fieldWidth2);
barSingle = repmat('-', 1, fieldWidth1 + fieldWidth2);

% re-organize the data
c = c(2:end,[2 1 3 4 5]);
c = [num2cell(repmat(fieldWidth1, size(c,1), 1)), effect(2:end), c]';

% create the ANOVA table
s = sprintf('ANOVA table for %s\n', dvName);
s = [s sprintf('%s\n', barDouble)];
s = [s sprintf('%-*s %4s %11s %14s %9s %9s\n', fieldWidth1, 'Effect', 'df', 'SS', 'MS', 'F', 'p')];
s = [s sprintf('%s\n', barSingle)];
s = [s sprintf('%-*s %4d %14.5f %14.5f %10.3f %10.4f\n', c{:})];
s = [s sprintf('%s\n', barDouble)];
s = [s sprintf('File: %s\n', dataFile)];

end

% *** end ***