function [z,p] = wilcoxonsignedranks(M)
% wilcoxonsignedranks - perform the Wilcoxon Signed-Ranks test on two
% columns of data
%
%   [z, p] = wilcoxonsignedranks(M) returns the test statistic (z) and the
%   probability (p) for the Wilcoxon Signed-Ranks test performed on the
%   first two columns of data in the matrix M. The values for z and p are
%   corrected for ties.
%
%   NOTE:  This test is sometimes called the Wilcoxon Matched-Pairs
%   Signed-Ranks test -- to distinguish it from a similar single-sample
%   Wilcoxon test.
%
%   The Wilcoxon Signed-Ranks test is a non-parametric test comparing two
%   sets of ordinal (i.e., ranked) data.  The two sets are 'dependent',
%   meaning they are from a group of participants with each participant
%   tested under both conditions.  This is also known as a within-subjects
%   design.

% The computations below are from pp. 810-817 in Sheskin's "Handbook of
% Parametric and Non-parametric Statistical Procedures", 5th ed., CRC
% Press, 2011.

% The second, corrected, z-score below is corrected for ties.  It does
% *not* include the correction for continuity described by Sheskin (pp.
% 815-816).

D = M(:,1) - M(:,2); % difference for each row
D = D(D ~= 0); % remove zeros (no difference)

[R, tieadj]  = tiedrank(abs(D));
n = length(R);

Rplus = sum(R(D > 0));
Rminus = sum(R(D < 0));

T = min([Rplus Rminus]);

% uncorrected
z1 = (T - (n*(n+1))/4) / sqrt((n*(n+1)*(2*n+1))/24);
p1 = 2*normcdf(z1);

% corrected for ties
z = (T - (n*(n+1))/4) / sqrt(((n*(n+1)*(2*n+1)) -  tieadj)/24);
p = 2*normcdf(z);

end